import * as assert from "assert";
import { env } from "$env/dynamic/private";

type RecordFromKeys<T extends readonly string[]> = Record<T[number], string>;

function ensureEnv<K extends readonly string[]>(keys: K): RecordFromKeys<K> {
  const cleanEnv = {} as RecordFromKeys<K>;

  for (const key of keys) {
    const value = env[key];
    if (!env.CI) {
      assert.ok(value, `Variable d'environnement ${key} manquante`);
    }
    cleanEnv[key as keyof RecordFromKeys<K>] = value!;
  }

  return cleanEnv;
}

export default ensureEnv([
  "API_ORIGIN",
  "API_TOKEN",
  "COOKIE_SECRET",
  "AIDC_CLIENT_ID",
  "AIDC_CLIENT_SECRET",
] as const);
