import { client } from "$lib/graphql";
import type { DocumentInput } from "@urql/core";
import { handleGqlError } from "./error";

type Page<T> = {
  page: {
    pageInfo: {
      hasNextPage: boolean;
      endCursor: string | null;
    };
    nodes: T[];
  };
};

interface PageArgs {
  first: number;
  after: string | null;
}

export async function* pageIterator<T, Variables extends PageArgs>(
  query: DocumentInput<Page<T>, Variables>,
  variables: Omit<Variables, "first" | "after">,
  pageSize = 100,
): AsyncGenerator<T, void, undefined> {
  let cursor: string | null = null;
  let hasNextPage = true;

  while (hasNextPage) {
    const result = await client
      .query(query, {
        first: pageSize,
        after: cursor,
        ...variables,
      } as Variables)
      .toPromise();

    handleGqlError(result);

    const page: Page<T>["page"] | undefined = result.data?.page;
    if (!page) return;

    yield* page.nodes ?? [];
    cursor = page.pageInfo.endCursor ?? null;
    hasNextPage = page.pageInfo.hasNextPage;
  }
}
